export type Collection<T extends {}> = TributeCollection<T> & TributeTemplateWithDefault<T>;
export interface ITribute<T extends {}> {
    autocompleteMode: boolean;
    autocompleteSeparator: RegExp | null;
    allowSpaces: boolean;
    closeOnScroll: boolean | HTMLElement;
    current: ITributeContext<T>;
    collection: Collection<T>[];
    menu: ITributeMenu<T>;
    hideMenu(): void;
    range: ITributeRange<T>;
    search: ITributeSearch<T>;
    hasTrailingSpace: boolean;
    isActive: boolean;
    menuContainer?: Element | null;
    hideMenu: () => void;
    positionMenu: boolean;
    replaceTextSuffix: string | null;
    showMenuFor(element: Element, scrollTo?: boolean): void;
    showMenuForCollection(element: HTMLElement, collectionIndex?: number): void;
    spaceSelectsMatch: boolean;
    selectItemAtIndex(index: string, originalEvent: Event): void;
    triggers(): (string | undefined)[];
}
export interface ITributeContext<T extends {}> {
    element?: HTMLElement;
    filteredItems?: TributeItem<T>[];
    collection?: Collection<T>;
    mentionText: string;
    isMentionLengthUnderMinimum: boolean | undefined;
    externalTrigger: boolean;
    process(scrollTo?: boolean): void;
    trigger?: string;
    updateSelection(info: TriggerInfo): void;
}
export type Coordinate = {
    top?: number | 'auto';
    left?: number | 'auto';
    right?: number | 'auto';
    bottom?: number | 'auto';
    height?: number;
    width?: number;
    maxHeight?: number;
    maxWidth?: number;
    position?: string;
};
export type TriggerInfo = {
    mentionPosition: number;
    mentionText: string | undefined;
    mentionSelectedElement: unknown;
    mentionSelectedPath: (number | undefined)[] | undefined;
    mentionSelectedOffset?: number;
    mentionTriggerChar?: string;
};
export interface ITributeMenu<T extends {}> {
    element: HTMLElement | null;
    activate(): void;
    deactivate(): void;
    isActive: boolean;
    create(doc: Document, containerClass: string): HTMLElement;
    getDimensions(): {
        height: number | null;
        width: number | null;
    };
    selected: number;
    setActiveLi(index: number): void;
    positionAtCaret(info: unknown, coordinates: Coordinate): void;
    up(count: number): void;
    down(count: number): void;
    unselect(): void;
}
export interface ITributeEvents {
    bind(element: HTMLElement): void;
    unbind(element: HTMLElement): void;
}
export interface ITributeRange<T extends {}> {
    getDocument(): Document;
    positionMenuAtCaret(scrollTo?: boolean): void;
    replaceTriggerText(text: string | HTMLElement, requireLeadingSpace: boolean, hasTrailingSpace: boolean, originalEvent: Event, item: TributeItem<T>): void;
    getTriggerInfo(menuAlreadyActive: boolean, hasTrailingSpace: boolean, requireLeadingSpace: boolean, allowSpaces: boolean, isAutocomplete: boolean): TriggerInfo | undefined;
}
export interface ITributeSearch<T extends {}> {
    filter(pattern: string, arr: T[], opts?: TributeSearchOpts<T>): TributeItem<T>[];
}
export type TributeItem<T extends {}> = {
    index: number;
    original: T;
    score: number;
    string: string;
};
export type TributeSearchOpts<T extends {}> = {
    pre?: string;
    post?: string;
    skip?: boolean;
    caseSensitive?: boolean;
    extract?: (element: T & {
        [key: string]: string;
    }) => string | null | undefined;
};
export type TributeCollection<T extends {}> = {
    trigger: string;
    iframe?: HTMLIFrameElement | null;
    shadowRoot?: (ShadowRoot & {
        getSelection(): Selection;
    }) | null;
    selectClass: string;
    containerClass: string;
    itemClass: string;
    menuContainer?: Element | null;
    lookup?: string | ((item: T, mentionText: string) => string);
    fillAttr: string;
    values: Array<T> | ((text: string, cb: (result: Array<T>) => void) => void) | null;
    loadingItemTemplate: string | null;
    requireLeadingSpace?: boolean;
    allowSpaces?: boolean;
    replaceTextSuffix?: string | null;
    positionMenu?: boolean;
    spaceSelectsMatch?: boolean;
    autocompleteMode?: boolean;
    searchOpts: TributeSearchOpts<T>;
    menuItemLimit?: number | null;
    menuShowMinLength: number;
    maxDisplayItems?: number | null;
    isBlocked?: boolean;
};
export type TributeTemplate<T extends {}> = {
    selectTemplate?: ((item: TributeItem<T> | undefined) => string | HTMLElement) | null;
    menuItemTemplate?: ((item: TributeItem<T>) => string | HTMLElement) | null;
    noMatchTemplate?: (() => string) | string | null;
};
export type TributeTemplateWithDefault<T extends {}> = {
    selectTemplate: (item: TributeItem<T> | undefined) => string | HTMLElement;
    menuItemTemplate: (item: TributeItem<T>) => string | HTMLElement;
    noMatchTemplate: (() => string) | string | null;
};
export type TributeArgument<T extends {}> = {
    autocompleteSeparator?: RegExp | null;
    closeOnScroll?: boolean | HTMLElement;
    collection: (TributeCollection<T> & TributeTemplate<T>)[] | null;
};
export type TributeElement = HTMLElement | NodeList | HTMLCollection | Array<HTMLElement>;
